use ratatui::style::{Style, Color};
use ratatui::widgets::block::BorderType;

#[derive(Debug, Clone)]
pub struct StylePalette {
    normal_style: Option<Style>,
    hover_style: Option<Style>,

    normal_border_style: Option<Style>,
    hover_border_style: Option<Style>,

    normal_title_style: Option<Style>,
    hover_title_style: Option<Style>,

    tagged_style: Option<Style>,

    normal_border_type: BorderType,
    hover_border_type: BorderType,
}

impl Default for StylePalette {
    fn default() -> Self {
        Self::new()
    }
}

impl StylePalette {
    pub fn new() -> Self {
        Self {
            normal_style: None,
            hover_style: None,
            normal_border_style: None,
            hover_border_style: None,
            normal_title_style: None,
            hover_title_style: None,
            tagged_style: None,

            normal_border_type: BorderType::Plain,
            hover_border_type: BorderType::Double,
        }
    }

    pub fn with_normal_style(&mut self, normal_style: Option<Style>) {
        self.normal_style = normal_style;
    }

    pub fn with_hover_style(&mut self, hover_style: Option<Style>) {
        self.hover_style = hover_style;
    }

    pub fn with_normal_border_style(&mut self, normal_border_style: Option<Style>) {
        self.normal_border_style = normal_border_style;
    }

    pub fn with_hover_border_style(&mut self, hover_border_style: Option<Style>) {
        self.hover_border_style = hover_border_style;
    }

    pub fn with_normal_title_style(&mut self, normal_title_style: Option<Style>) {
        self.normal_title_style = normal_title_style;
    }

    pub fn with_hover_title_style(&mut self, hover_title_style: Option<Style>) {
        self.hover_title_style = hover_title_style;
    }

    pub fn with_tagged_style(&mut self, tagged_style: Option<Style>) {
        self.tagged_style = tagged_style;
    }

    pub fn create_tagged_style(&self) -> Style {
        self.tagged_style.unwrap_or_default()
    }

    pub fn create_text_style(&mut self, active: bool) -> Style {
        if active { 
            self.hover_style.unwrap_or_default()
        } else { 
            self.normal_style.unwrap_or_default()
        }
    }

    pub fn create_border_style(&self, active: bool) -> (Color, BorderType) {
        if active {
            (
                self.hover_border_style.map(|style| style.fg).flatten().unwrap_or_default(),
                self.hover_border_type, 
            )
        } else {
            (
                self.normal_border_style.map(|style| style.fg).flatten().unwrap_or_default(),
                self.normal_border_type, 
            )
        }
    }

    pub fn create_title_style(&self, active: bool) -> Style {
        if active {
            self.normal_title_style.unwrap_or_default()
        } else {
            self.hover_title_style.unwrap_or_default()
        }
    }
}
